@@ -25,6 +25,7 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 
 #include <glib.h>
 
@@ -63,6 +64,21 @@ clean_dir (const gchar *dirname)
 	     dirname, strerror (errno));
 }
 
+void read_gstring_from_stdin(GString *s)
+{
+  gchar buffer[255];
+  char *readp;
+  do
+  {
+     readp = fgets(buffer, sizeof(buffer), stdin);
+     if(readp == NULL)
+	return;
+     strip (buffer);
+     g_string_append(s, buffer);
+  } while (sizeof(buffer)-1 == strlen(readp));
+  return;
+}
+
 int
 main (int argc, char **argv)
 {
@@ -79,8 +95,6 @@ main (int argc, char **argv)
   gchar *xauth_token = NULL;
   gchar *sn_id = NULL;
 
-  gchar buffer[255];
-
   gint return_code;
 
   if (argc < 2)
@@ -101,24 +115,28 @@ main (int argc, char **argv)
   xauth_file = g_strdup_printf ("%s/.Xauthority",
 				xauth_dir);
 
-  fgets (buffer, 255, stdin);
-  strip (buffer);
+  GString *s = g_string_sized_new(255);
+  read_gstring_from_stdin(s);
 
   /* strlen ("gksu-run: ") == 10, see su.c */
-  xauth_display = g_strdup_printf ("%s", buffer + 10);
+  xauth_display = g_strdup_printf ("%s", s->str + 10);
 
-  bzero (buffer, 255);
-  fgets (buffer, 255, stdin);
-  strip (buffer);
+  s = g_string_truncate(s,0);
+  read_gstring_from_stdin(s);
 
-  sn_id = g_strdup_printf ("%s", buffer + 10);
+  sn_id = g_strdup_printf ("%s", s->str + 10);
   setenv("DESKTOP_STARTUP_ID", sn_id, TRUE);
 
-  bzero (buffer, 255);
-  fgets (buffer, 255, stdin);
-  strip (buffer);
+  /* cleanup the environment; some variables we bring from the user
+   * environment make gconf-based applications misbehaving these days
+   */
+  unsetenv ("ORBIT_SOCKETDIR");
+  unsetenv ("DBUS_SESSION_BUS_ADDRESS");
 
-  xauth_token = g_strdup_printf ("%s", buffer + 10);
+  s = g_string_truncate(s,0);
+  read_gstring_from_stdin(s);
+
+  xauth_token = g_strdup_printf ("%s", s->str + 10);
 
   /* a bit more security is always fine */
   {
@@ -161,6 +179,12 @@ main (int argc, char **argv)
     return_code = system (argv[1]);
     
     clean_dir (xauth_dir);
-    return return_code;
+    g_string_free(s, TRUE);
+    if (WIFEXITED(return_code))
+    {
+      return WEXITSTATUS(return_code);
+    } else if (WIFSIGNALED(return_code)) {
+      return -1;
+    }
   }
 }
