@@ -1,6 +1,6 @@
 /*
  * Gksu -- a library providing access to su functionality
- * Copyright (C) 2004 Gustavo Noronha Silva
+ * Copyright (C) 2004-2009 Gustavo Noronha Silva
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -533,9 +533,13 @@ grab_keyboard_and_mouse (GtkWidget *dialog)
   GdkGrabStatus status;
   gint grab_tries = 0;
   gint lock = -1;
+  pid_t pid;
 
-  gchar *fname = g_strdup_printf ("%s/.gksu.lock", getenv ("HOME"));
-  pid_t pid = test_lock (fname);
+  gchar *fname = g_strdup (getenv ("GKSU_LOCK_FILE"));
+  if (fname == NULL)
+    fname = g_strdup_printf ("%s/.gksu.lock", getenv ("HOME"));
+
+  pid = test_lock (fname);
 
   if (pid != 0)
     {
@@ -548,6 +552,7 @@ grab_keyboard_and_mouse (GtkWidget *dialog)
     g_warning ("Unable to create lock file.");
   g_free (fname);
 
+  gdk_threads_enter ();
   fadeout_screen (gdk_screen_get_default (), 0);
   gtk_widget_show_all (dialog);
 
@@ -614,6 +619,7 @@ ungrab_keyboard_and_mouse (int lock)
 
   g_get_current_time (&fade_data->start_time);
   while (fadein_callback (fade_data) != FALSE);
+  gdk_threads_leave();
 
   close(lock);
 }
@@ -1715,7 +1721,7 @@ static void
 gksu_context_launch_initiate (GksuContext *context)
 {
   gchar *sid = NULL;
-  guint32 launch_time = gdk_x11_display_get_user_time ((GdkDisplay*)GDK_DISPLAY());
+  guint32 launch_time = gdk_x11_display_get_user_time (gdk_display_get_default ());
   static gboolean initiated = FALSE;
 
   if (!initiated)
@@ -1846,8 +1852,6 @@ gksu_context_get_type (void)
 }
 
 
-
-
 /**
  * gksu_su_full:
  * @context: a #GksuContext
@@ -1864,6 +1868,47 @@ gksu_context_get_type (void)
  * #GksuPasswordNotNeededFunc
  * @error: a #GError object to be filled with the error code or NULL
  *
+ * This is a compatibility shim over gksu_su_fuller, which, for
+ * compatibility reasons, lacks the 'exit_status' argument. You should
+ * check the documentation for gksu_su_fuller for information about
+ * the arguments.
+ *
+ * Returns: TRUE if all went fine, FALSE if failed
+ */
+
+gboolean
+gksu_su_full (GksuContext *context,
+	      GksuAskPassFunc ask_pass,
+	      gpointer ask_pass_data,
+	      GksuPassNotNeededFunc pass_not_needed,
+	      gpointer pass_not_needed_data,
+	      GError **error)
+{
+  return gksu_su_fuller(context,
+  			ask_pass, ask_pass_data,
+			pass_not_needed, pass_not_needed_data,
+			NULL, error);
+}
+
+
+/**
+ * gksu_su_fuller:
+ * @context: a #GksuContext
+ * @ask_pass: a #GksuAskPassFunc to be called when the lib determines
+ * requesting a password is necessary; it may be NULL, in which case
+ * the standard password request dialog will be used
+ * @ask_pass_data: a #gpointer with user data to be passed to the
+ * #GksuAskPasswordFunc
+ * @pass_not_needed: a #GksuPassNotNeededFunc that will be called
+ * when the command is being run without the need for requesting
+ * a password; it will only be called if the display-no-pass-info
+ * gconf key is enabled; NULL will have the standard dialog be shown
+ * @pass_not_needed_data: a #gpointer with the user data to be passed to the
+ * #GksuPasswordNotNeededFunc
+ * @exit_status: an optional pointer to a #gint8 which will be filled with
+ * the exit status of the child process
+ * @error: a #GError object to be filled with the error code or NULL
+ *
  * This could be considered one of the main functions in GKSu.
  * it is responsible for doing the 'user changing' magic calling
  * the #GksuAskPassFunc function to request a password if needed.
@@ -1876,12 +1921,13 @@ gksu_context_get_type (void)
  * Returns: TRUE if all went fine, FALSE if failed
  */
 gboolean
-gksu_su_full (GksuContext *context,
-	      GksuAskPassFunc ask_pass,
-	      gpointer ask_pass_data,
-	      GksuPassNotNeededFunc pass_not_needed,
-	      gpointer pass_not_needed_data,
-	      GError **error)
+gksu_su_fuller (GksuContext *context,
+	        GksuAskPassFunc ask_pass,
+	        gpointer ask_pass_data,
+	        GksuPassNotNeededFunc pass_not_needed,
+	        gpointer pass_not_needed_data,
+	        gint8 *exit_status,
+	        GError **error)
 {
   GQuark gksu_quark;
   int i = 0;
@@ -2253,6 +2299,15 @@ gksu_su_full (GksuContext *context,
       if (pid_exited != pid)
 	waitpid(pid, &status, 0);
 
+      if (exit_status)
+      {
+      	if (WIFEXITED(status)) {
+      	  *exit_status = WEXITSTATUS(status);
+	} else if (WIFSIGNALED(status)) {
+	  *exit_status = -1;
+	}
+      }
+
       if (WEXITSTATUS(status))
 	{
 	  if(cmdline)
@@ -2343,6 +2398,46 @@ read_line (int fd, gchar *buffer, int n)
  * #GksuPasswordNotNeededFunc
  * @error: a #GError object to be filled with the error code or NULL
  *
+ * This is a compatibility shim over gksu_sudo_fuller, which, for
+ * compatibility reasons, lacks the 'exit_status' argument. You should
+ * check the documentation for gksu_sudo_fuller for information about
+ * the arguments.
+ *
+ * Returns: TRUE if all went fine, FALSE if failed
+ */
+
+gboolean
+gksu_sudo_full (GksuContext *context,
+		GksuAskPassFunc ask_pass,
+		gpointer ask_pass_data,
+		GksuPassNotNeededFunc pass_not_needed,
+		gpointer pass_not_needed_data,
+		GError **error)
+{
+  return gksu_sudo_fuller(context,
+  			  ask_pass, ask_pass_data,
+			  pass_not_needed, pass_not_needed_data,
+			  NULL, error);
+}
+
+/**
+ * gksu_sudo_fuller:
+ * @context: a #GksuContext
+ * @ask_pass: a #GksuAskPassFunc to be called when the lib determines
+ * requesting a password is necessary; it may be NULL, in which case
+ * the standard password request dialog will be used
+ * @ask_pass_data: a #gpointer with user data to be passed to the
+ * #GksuAskPasswordFunc
+ * @pass_not_needed: a #GksuPassNotNeededFunc that will be called
+ * when the command is being run without the need for requesting
+ * a password; it will only be called if the display-no-pass-info
+ * gconf key is enabled; NULL will have the standard dialog be shown
+ * @pass_not_needed_data: a #gpointer with the user data to be passed to the
+ * #GksuPasswordNotNeededFunc
+ * @error: a #GError object to be filled with the error code or NULL
+ * @exit_status: an optional pointer to a #gint8 which will be filled with
+ * the exit status of the child process
+ *
  * This could be considered one of the main functions in GKSu.
  * it is responsible for doing the 'user changing' magic calling
  * the #GksuAskPassFunc function to request a password if needed.
@@ -2355,12 +2450,13 @@ read_line (int fd, gchar *buffer, int n)
  * Returns: TRUE if all went fine, FALSE if failed
  */
 gboolean
-gksu_sudo_full (GksuContext *context,
-		GksuAskPassFunc ask_pass,
-		gpointer ask_pass_data,
-		GksuPassNotNeededFunc pass_not_needed,
-		gpointer pass_not_needed_data,
-		GError **error)
+gksu_sudo_fuller (GksuContext *context,
+		  GksuAskPassFunc ask_pass,
+		  gpointer ask_pass_data,
+		  GksuPassNotNeededFunc pass_not_needed,
+		  gpointer pass_not_needed_data,
+		  gint8 *exit_status,
+		  GError **error)
 {
   char **cmd;
   char buffer[256] = {0};
@@ -2634,6 +2730,8 @@ gksu_sudo_full (GksuContext *context,
 
 	  if (context->debug)
 	    fprintf (stderr, "buffer: -%s-\n", buffer);
+
+	  usleep(1000);
 	}
 
       if (context->debug)
@@ -2753,6 +2851,15 @@ gksu_sudo_full (GksuContext *context,
 	waitpid(pid, &status, 0);
       sudo_reset_xauth (context, xauth, xauth_env);
 
+      if (exit_status)
+      {
+      	if (WIFEXITED(status)) {
+      	  *exit_status = WEXITSTATUS(status);
+	} else if (WIFSIGNALED(status)) {
+	  *exit_status = -1;
+	}
+      }
+
       if (WEXITSTATUS(status))
 	{
 	  if(cmdline)
@@ -2829,6 +2936,44 @@ gksu_sudo (gchar *command_line,
  * #GksuPasswordNotNeededFunc
  * @error: a #GError object to be filled with the error code or NULL
  *
+ * This is a compatibility shim over gksu_run_fuller, which, for
+ * compatibility reasons, lacks the 'exit_status' argument.
+ *
+ * Returns: TRUE if all went fine, FALSE if failed
+ */
+
+gboolean
+gksu_run_full (GksuContext *context,
+	       GksuAskPassFunc ask_pass,
+	       gpointer ask_pass_data,
+	       GksuPassNotNeededFunc pass_not_needed,
+	       gpointer pass_not_needed_data,
+	       GError **error)
+{
+  return gksu_run_fuller(context,
+  			 ask_pass, ask_pass_data,
+			 pass_not_needed, pass_not_needed_data,
+			 NULL, error);
+}
+
+/**
+ * gksu_run_fuller:
+ * @context: a #GksuContext
+ * @ask_pass: a #GksuAskPassFunc to be called when the lib determines
+ * requesting a password is necessary; it may be NULL, in which case
+ * the standard password request dialog will be used
+ * @ask_pass_data: a #gpointer with user data to be passed to the
+ * #GksuAskPasswordFunc
+ * @pass_not_needed: a #GksuPassNotNeededFunc that will be called
+ * when the command is being run without the need for requesting
+ * a password; it will only be called if the display-no-pass-info
+ * gconf key is enabled; NULL will have the standard dialog be shown
+ * @pass_not_needed_data: a #gpointer with the user data to be passed to the
+ * #GksuPasswordNotNeededFunc
+ * @exit_status: an optional pointer to a #gint8 which will be filled with
+ * the exit status of the child process
+ * @error: a #GError object to be filled with the error code or NULL
+ *
  * This function is a wrapper for gksu_sudo_full/gksu_su_full. It will
  * call one of them, depending on the GConf key that defines whether
  * the default behavior for gksu is su or sudo mode. This is the
@@ -2837,12 +2982,13 @@ gksu_sudo (gchar *command_line,
  * Returns: TRUE if all went fine, FALSE if failed
  */
 gboolean
-gksu_run_full (GksuContext *context,
-	       GksuAskPassFunc ask_pass,
-	       gpointer ask_pass_data,
-	       GksuPassNotNeededFunc pass_not_needed,
-	       gpointer pass_not_needed_data,
-	       GError **error)
+gksu_run_fuller (GksuContext *context,
+	         GksuAskPassFunc ask_pass,
+	         gpointer ask_pass_data,
+	         GksuPassNotNeededFunc pass_not_needed,
+	         gpointer pass_not_needed_data,
+		 gint8 *exit_status,
+	         GError **error)
 {
   GConfClient *gconf_client;
   gboolean sudo_mode;
@@ -2853,13 +2999,13 @@ gksu_run_full (GksuContext *context,
   g_object_unref (gconf_client);
 
   if (sudo_mode)
-    return gksu_sudo_full (context, ask_pass, ask_pass_data,
-			   pass_not_needed, pass_not_needed_data,
-			   error);
+    return gksu_sudo_fuller (context, ask_pass, ask_pass_data,
+			     pass_not_needed, pass_not_needed_data,
+			     exit_status, error);
 
-  return gksu_su_full (context, ask_pass, ask_pass_data,
-		       pass_not_needed, pass_not_needed_data,
-		       error);
+  return gksu_su_fuller (context, ask_pass, ask_pass_data,
+		         pass_not_needed, pass_not_needed_data,
+		         exit_status, error);
 }
 
 /**
